import json
import re
import sys
import os
import collections.abc

class JsonTemplates:

#-----------------------------------------------------------------------
 def __init__(self): 
  self.__template = None
  self.__var_regex = re.compile("\{\{\ [a-zA-Z0-9_]+\ \}\}")
  self.__arr_regex = re.compile("\{\%\ [a-zA-Z0-9_]+\ \%\}")
  self.__cln_regex = re.compile("[a-zA-Z0-9_]+")
  self.__version__ = "0.1.0"

#-----------------------------------------------------------------------
 def is_dict(self,obj):
  return isinstance(obj,collections.abc.Mapping)
   
#-----------------------------------------------------------------------
 def is_iterable(self,obj):
  try:
   iter(obj)
   return True
  except:
   return False

#-----------------------------------------------------------------------
 def clean_key(self,key):
  mtch = self.__cln_regex.search(key)
  if mtch is None:
   return (False,"{} is not valid!".format(key))
  else:
   return (True,mtch.group(0))

#-----------------------------------------------------------------------
 def loads(self,json_str):
  try:
   self.__template = json.loads(json_str)
  except Exception as ex:
   return (False, "Unable to parse json! {}".format(ex))

  return (True,self.__template)

#-----------------------------------------------------------------------
 def load(self,json_path):
  if os.path.isfile(json_path) and json_path.lower().endswith("json"):
   try:
    with open(json_path) as json_in:
     self.__template = json.load(json_in)
   except Exception as ex:
    return (False, "Unable to parse json! {}".format(ex))
  else:
   return (False, "{} is not JSON file")
  return (True,self.__template)
 
#-----------------------------------------------------------------------
 def __evalulate_value(self,value,replace_dict):
  not_found_error = "{} not found in replacement dictionary {}"
  if self.__var_regex.fullmatch(str(value)):
   replace_key = self.clean_key(value)
   if not replace_key[0]:
    return (False,False,replace_key[1])
   replace_key = replace_key[1]
   if replace_key in replace_dict:
    return (True,True,replace_dict[replace_key])
   else:
    return (False,False,not_found_error.format(replace_key,replace_dict))
  elif self.__arr_regex.fullmatch(str(value)):
   replace_key = self.clean_key(value)
   if not replace_key[0]:
    return (False,False,replace_key[1])
   replace_key = replace_key[1]
   
   if replace_key in replace_dict:
    if self.is_iterable(replace_dict[replace_key]):
     return (True,True,replace_dict[replace_key])
    else:
     return (False,False,"{} value must be iterable".format(replace_key))
   else:

    return (False,False,not_found_error.format(replace_key,replace_dict))
   
  else:
   return (True,False,value)
   
#-----------------------------------------------------------------------
 def __scan_nodes(self,nodes,replace_dict):
  for k,v in nodes.items():
   if self.is_dict(v):
    nodes[k] = self.__scan_nodes(v,replace_dict)
   else:
    replace_result = self.__evalulate_value(v,replace_dict)
    if replace_result[0]:
     nodes[k] = replace_result[2]
    else:
     raise Exception("Unable to find a replacement value for '{}. {}'".format(k,replace_result[2]))
  return nodes
  
#-----------------------------------------------------------------------
 def generate(self,replace_dict):

  try:
   result = self.__scan_nodes(dict(self.__template),replace_dict)
  except Exception as ex:
   return (False,"Error: {}".format(ex))
  return (True,result)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="jsontemplates",
    version="0.1.0",
    packages=find_packages(),
    install_requires=[],
    author='Jon Keatley',
    description='A small python module for populating json template files.',
    url='https://gitlab.com/Jon.Keatley.Folio/json-templates',
    project_urls={'Source Code':'https://gitlab.com/Jon.Keatley.Folio/json-templates'},
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires='>=3.5',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import os
import sys
import unittest 
from json_templates import *

GOOD_JSON = """{
   "key":"hard coded value",
   "key2":1,
   "key3":"{{ variable }}",
   "key4":"{% array %}"
}
"""

BAD_JSON = """{
   "key":"hard coded value",
   "key2":1,
   "key3":"{{ variable }}"
   "key4":"{% array %}"
}
"""

REPLACE_TEMPLATE = """{
 "key":"{{ key }}"
}"""

REPLACE_RESULT = "hello world"
REPLACE_KEY    = "key"

REPLACE_ARR_TEMPLATE = """{
 "array":"{% array %}"
}"""

REPLACE_ARR_RESULT = ["hello","world"]
REPLACE_ARR_KEY    = "array"

NESTING_JSON = """{
 "key":{
  "innerkey":{
   "inner2key":{
    "inner3key1":"a value",
    "inner3key2":4,
    "inner3key3":"{{ innertest }}"
   }
  }
 }
}"""

NESTING_KEY = "innertest"
NESTING_VALUE = "success"

class basic_tests(unittest.TestCase):
 
#-----------------------------------------------------------------------
 def test_object(self):
  json_tmp = JsonTemplates()
  self.assertIsInstance(json_tmp,JsonTemplates)

 
#-----------------------------------------------------------------------
 def test_json_string(self):
  json_tmp = JsonTemplates()
  self.assertTrue(json_tmp.loads(GOOD_JSON)[0])
  
#-----------------------------------------------------------------------
 def test_json_file(self):
  json_tmp = JsonTemplates()
  result = json_tmp.load(os.path.join(os.getcwd(),"examples/example1.json"))
  self.assertTrue(result[0],msg="Failed to load json file. Error: {}".format(result[1]))
  
#-----------------------------------------------------------------------
 def test_bad_json_string(self):
  json_tmp = JsonTemplates()
  self.assertFalse(json_tmp.loads(BAD_JSON)[0])

#-----------------------------------------------------------------------
 def test_compare_variable_replacement(self):
  json_tmp = JsonTemplates()
  self.assertTrue(json_tmp.loads(REPLACE_TEMPLATE)[0])
  
  result = json_tmp.generate({REPLACE_KEY:REPLACE_RESULT})
  
  self.assertTrue(result[0],msg="Generate failed with this message. {}".format(result[1]))

  self.assertEqual(result[1][REPLACE_KEY],REPLACE_RESULT)
  
#-----------------------------------------------------------------------
 def test_compare_array_replacement(self):
  json_tmp = JsonTemplates()
  self.assertTrue(json_tmp.loads(REPLACE_ARR_TEMPLATE)[0])
  
  result = json_tmp.generate({REPLACE_ARR_KEY:REPLACE_ARR_RESULT})
  
  self.assertTrue(result[0],msg="Generate failed with this message. {}".format(result[1]))
  
  for x in range(0,len(REPLACE_ARR_RESULT)):
   self.assertEqual(result[1][REPLACE_ARR_KEY][x],REPLACE_ARR_RESULT[x])
   
#-----------------------------------------------------------------------
 def test_compare_nesting_replacement(self):
  json_tmp = JsonTemplates()
  self.assertTrue(json_tmp.loads(NESTING_JSON)[0])
  
  result = json_tmp.generate({NESTING_KEY:NESTING_VALUE})
  
  self.assertTrue(result[0],msg="Generate failed with this message. {}".format(result[1]))
  
  self.assertEqual(result[1]["key"]["innerkey"]["inner2key"]["inner3key3"],NESTING_VALUE)
 
#-----------------------------------------------------------------------
if __name__ == '__main__':
    unittest.main()
